@@ -1,10 +1,8 @@
+#!/usr/bin/perl
+
 package Net::MAC::Vendor;
 use strict;
 
-use v5.10;
-
-=encoding utf8
-
 =head1 NAME
 
 Net::MAC::Vendor - look up the vendor for a MAC
@@ -34,33 +32,19 @@ This module allows you to take a MAC address and turn it into the OUI
 and vendor information. You can, for instance, scan a network,
 collect MAC addresses, and turn those addresses into vendors. With
 vendor information, you can often guess at what what you are looking
-at (I<e.g.> an Apple product).
+at (e.g. an Apple product).
 
 You can use this as a module as its individual functions, or call it
 as a script with a list of MAC addresses as arguments. The module can
 figure it out.
 
-The IEEE moves the location of its OUI file. If they do that again, you
-can set the C<NET_MAC_VENDER_OUI_URL> environment variable to get the new
-URL without updating the code.
-
-Here are some of the old URLs, which also flip-flop schemes:
-
-	http://standards.ieee.org/regauth/oui/oui.txt
-	https://standards.ieee.org/regauth/oui/oui.txt
-	http://standards-oui.ieee.org/oui.txt
-
-There are older copies of the OUI file in the GitHub repository.
-
-These files are large (about 4MB), so you might want to cache a copy.
-
-A different source of information is linuxnet.ca that publishes sanitized
-and compressed versions of the list, such as:
-
-        http://linuxnet.ca/ieee/oui.txt.bz2
-
-The module can read and decompress compressed versions (as long as the url
-reflects the compression type in the filename as the linuxnet.ca links do).
+This module tries to persistently cache with DBM::Deep the OUI
+information so it can avoid using the network. If it cannot load
+DBM::Deep, it uses a normal hash (which is lost when the process
+finishes). You can preload this cache with the C<load_cache()> function.
+So far, the module looks in the current working directory for a file
+named mac_oui.db to find the cache. I need to come up with a way to
+let the user set that location.
 
 =head2 Functions
 
@@ -68,17 +52,18 @@ reflects the compression type in the filename as the linuxnet.ca links do).
 
 =cut
 
-use Exporter qw(import);
+use base qw(Exporter);
 
 __PACKAGE__->run( @ARGV ) unless caller;
 
 use Carp;
-use Compress::Bzip2 qw(memBunzip);
-use Compress::Zlib  qw(memGunzip);
-use Mojo::URL;
-use Mojo::UserAgent;
+use LWP::Simple qw(get);
 
-our $VERSION = '1.260_03';
+# http://standards.ieee.org/regauth/oui/oui.txt
+
+our $Cached = {};
+
+our $VERSION = 1.19_01;
 
 =item run( @macs )
 
@@ -100,25 +85,11 @@ sub run {
 
 	foreach my $arg ( @_ ) {
 		my $lines = lookup( $arg );
-		return unless defined $lines;
 
 		unshift @$lines, $arg;
 
 		print join "\n", @$lines, '';
 		}
-
-	return 1;
-	}
-
-=item ua
-
-Return the Mojo::UserAgent object used to fetch resources.
-
-=cut
-
-sub ua {
-	state $ua = Mojo::UserAgent->new->max_redirects(3);
-	$ua;
 	}
 
 =item lookup( MAC )
@@ -128,9 +99,6 @@ information. The first element is the vendor name, and the remaining
 elements are the address lines. Different records may have different
 numbers of lines, although the first two should be consistent.
 
-This makes a direct request to the IEEE website for that OUI to return
-the information for that vendor.
-
 The C<normalize_mac()> function explains the possible formats
 for MAC.
 
@@ -207,43 +175,10 @@ To avoid multiple calls on the network, use C<load_cache> to preload
 the entire OUI space into an in-memory cache. This can take a long
 time over a slow network, though; the file is about 60,000 lines.
 
-Also, the IEEE website has been flaky lately, so loading the cache is
-better. This distribution comes with several versions of the complete
-OUI data file.
-
 =cut
 
 sub fetch_oui {
-	# fetch_oui_from_custom( $_[0] )    ||
-		fetch_oui_from_cache( $_[0] ) ||
-		fetch_oui_from_ieee( $_[0] );
-	}
-
-=item fetch_oui_from_custom( MAC, [ URL ] )
-
-Looks up the OUI information from the specified URL or the URL set
-in the C<NET_MAC_VENDOR_OUI_SOURCE> environment variable.
-
-The C<normalize_mac()> function explains the possible formants for
-MAC.
-
-=cut
-
-sub fetch_oui_from_custom {
-	my $mac = normalize_mac( shift );
-	my $url = shift // $ENV{NET_MAC_VENDOR_OUI_SOURCE};
-
-	return unless defined $url;
-
-	my $html = __PACKAGE__->ua->get( $url );
-	unless( defined $html ) {
-		carp "Could not fetch data from the IEEE!";
-		return;
-		}
-
-	parse_oui(
-		extract_oui_from_html( $html, $mac )
-		);
+	fetch_oui_from_cache( $_[0] ) || fetch_oui_from_ieee( $_[0] );
 	}
 
 =item fetch_oui_from_ieee( MAC )
@@ -256,34 +191,10 @@ MAC.
 
 =cut
 
-sub _search_url_base {
-# https://services13.ieee.org/RST/standards-ra-web/rest/assignments/download/?registry=MA-L&format=html&text=00-0D-93
-	state $url = Mojo::URL->new(
-		'https://services13.ieee.org/RST/standards-ra-web/rest/assignments/download/?registry=MA-L&format=html'
-		);
-
-	$url;
-	}
-
-sub _search_url {
-	my( $class, $mac ) = @_;
-	my $url = $class->_search_url_base->clone;
-	$url->query->merge( text => $mac );
-	$url;
-	}
-
 sub fetch_oui_from_ieee {
 	my $mac = normalize_mac( shift );
 
-	my @urls = __PACKAGE__->_search_url( $mac );
-
-	my $html;
-	URL: foreach my $url ( @urls ) {
-		$html = __PACKAGE__->_fetch_oui_from_url( $url );
-		next URL unless defined $html;
-		last;
-		}
-
+	my $html = get( "http://standards.ieee.org/cgi-bin/ouisearch?$mac" );
 	unless( defined $html ) {
 		carp "Could not fetch data from the IEEE!";
 		return;
@@ -294,35 +205,6 @@ sub fetch_oui_from_ieee {
 		);
 	}
 
-sub _fetch_oui_from_url {
-	my( $class, $url ) = @_;
-	my $tries = 0;
-
-	return unless defined $url;
-
-	TRY: {
-		my $tx = __PACKAGE__->ua->get( $url );
-		unless( $tx->success ) {
-			if( $tries > 3 ) {
-				carp "Failed fetching [$url]: " . $tx->res->code;
-				return;
-				}
-
-			$tries++;
-			sleep 1 * $tries;
-			redo TRY;
-			}
-
-		my $html = $tx->res->body;
-		unless( defined $html ) {
-			carp "No content in response for [$url]!";
-			return;
-			}
-
-		return $html;
-		}
-	}
-
 =item fetch_oui_from_cache( MAC )
 
 Looks up the OUI information in the cached OUI information (see
@@ -341,7 +223,7 @@ If it doesn't find the MAC in the cache, it returns nothing.
 sub fetch_oui_from_cache {
 	my $mac = normalize_mac( shift );
 
-	__PACKAGE__->get_from_cache( $mac );
+	exists $Cached->{ $mac } ? $Cached->{ $mac } : ();
 	}
 
 =item extract_oui_from_html( HTML, OUI )
@@ -359,16 +241,36 @@ sub extract_oui_from_html {
 	my $html = shift;
 	my $lookup_mac = normalize_mac( shift );
 
-	my( $record ) = $html =~ m|<pre>(<b>$lookup_mac</b>.*?)</pre>|is;
-	$record =~ s|</?b>||g;
+	$html =~ s/<pre>.*?$lookup_mac/<pre>$lookup_mac/is;
 
-	return unless defined $record;
-	return $record;
+	# sometimes the HTML returns more than one OUI because
+	# IEEE has a problem parsing their own data when they
+	# have private blocks
+	my( $ouis ) = $html =~ m|<pre>(.*?)</pre>|gs;
+	return unless defined $ouis;
+	$ouis =~ s/<\/?b>//gs; # remove bold around the OUI
+
+	my @entries = split /\n\s*\n/, $ouis;
+	return unless defined $entries[0];
+	return $entries[0] unless defined $entries[1];
+
+	my $result = $entries[0];
+
+	foreach my $entry ( @entries ) {
+		$entry =~ s/^\s+|\s+$//;
+		my $found_mac = normalize_mac( substr $entry, 0, 8 );
+		if( $found_mac eq $lookup_mac ) {
+			$result = $entry;
+			last;
+			}
+		}
+
+	return $result;
 	}
 
 =item parse_oui( STRING )
 
-Takes a string that looks like this:
+Takes a string that looks like
 
 	00-03-93   (hex)            Apple Computer, Inc.
 	000393     (base 16)        Apple Computer, Inc.
@@ -387,111 +289,67 @@ With no arguments, it returns an empty anonymous array.
 sub parse_oui {
 	my $oui = shift;
 	return [] unless $oui;
-	$oui =~ s|</?b>||g;
-	my @lines = map { s/^\s+//; $_ ? $_ : () } split /\s*$/m, $oui;
-	chomp @lines;
-	splice @lines, 1, 1, (); # should have documented this!
+
+	my @lines = map { s/^\s+//; $_ ? $_ : () } split /$/m, $oui;
+	splice @lines, 1, 1, ();
 
 	$lines[0] =~ s/\S+\s+\S+\s+//;
 	return \@lines;
 	}
 
-=item oui_url
+=item load_cache( [ SOURCE ] )
 
-=item oui_urls
+Downloads the current list of all OUIs, parses it with C<parse_oui()>,
+and stores it in C<$Cached> anonymous hash keyed by the OUIs (i.e.
+00-0D-93). The C<fetch_oui()> will use this cache if it exists.
 
-Returns the URLs of the oui.txt resource. The IEEE likes to move this
-around. These are the default URL that C<load_cache> will use, but you
-can also supply your own with the C<NET_MAC_VENDOR_OUI_URL> environment
-variable.
-
-=cut
-
-sub oui_url { (grep { /\Ahttp:/ } &oui_urls)[0] }
-
-sub oui_urls {
-	my @urls = 'http://standards-oui.ieee.org/oui.txt';
-
-	unshift @urls, $ENV{NET_MAC_VENDOR_OUI_URL}
-		if defined $ENV{NET_MAC_VENDOR_OUI_URL};
-
-	@urls;
-	}
-
-=item load_cache( [ SOURCE[, DEST ] ] )
-
-Downloads the current list of all OUIs in SOURCE, parses it with C<parse_oui()>,
-and stores it in the cache. The C<fetch_oui()> will use this cache if it exists.
-
-By default, this uses the URL from C<oui_url>,
+By default, this uses C<http://standards.ieee.org/regauth/oui/oui.txt>, 
 but given an argument, it tries to use that. To load from a local
 file, use the C<file://> scheme.
 
-If the url indicates that the data is compressed, the response content is
-decompressed before being stored.
-
 If C<load_cache> cannot load the data, it issues a warning and returns
 nothing.
 
 This previously used DBM::Deep if it was installed, but that was much
-too slow. Instead, if you want persistence, you can play with
+too slow. Instead, if you want persistence, you can play with 
 C<$Net::MAC::Vendor::Cached> yourself.
 
-If you want to store the data fetched for later use, add a destination
-filename to the request. To fetch from the default location and store,
-specify C<undef> as source.
-
 =cut
 
 sub load_cache {
-	my( $source, $dest ) = @_;
+	my $source = shift || "http://standards.ieee.org/regauth/oui/oui.txt";
 
-	my $data = do {;
-		if( defined $source ) {
-			unless( -e $source ) {
-				carp "Net::Mac::Vendor cache source [$source] does not exist";
+	my $data = do {
+		if( -e $source ) { # local files
+			do { local( @ARGV, $/ ) = $source; <> }
+			}
+		else { # everything else
+			my $data = get( $source );
+
+			unless( defined $data ) {
+				carp "Could not read from '$source'";
 				return;
 				}
 
-			do { local( @ARGV, $/ ) = $source; <> }
-			}
-		else {
-			#say time . " Fetching URL";
-			my $url = oui_url();
-			my $tx = __PACKAGE__->ua->get( $url );
-			#say time . " Fetched URL";
-			#say "size is " . $tx->res->headers->header( 'content-length' );
-			($url =~ /\.bz2/) ? memBunzip($tx->res->body) :
-			($url =~ /\.gz/)  ? memGunzip($tx->res->body) :
-			                    $tx->res->body;
+			$data;
 			}
 		};
 
-	if( defined $dest ) {
-		if( open my $fh, '>:utf8', $dest ) {
-			print { $fh } $data;
-			close $fh;
-			}
-		else { # notify on error, but continue
-			carp "Could not write to '$dest': $!";
-			}
-		}
-
-
 	# The PRIVATE entries fill in a template with no
 	# company name or address, but the whitespace is
 	# still there. We need to split on a newline
 	# followed by some potentially horizontal whitespace
 	# and another newline
-	my $CRLF = qr/(?:\r?\n)/;
-	my @entries = split /[\t ]* $CRLF [\t ]* $CRLF/x, $data;
+	my @entries = split /[\t ]*\n[\t ]*\n/, $data;
 	shift @entries;
 
 	my $count = '';
 	foreach my $entry ( @entries ) {
+	#	print STDERR "Processing ", ++$count, " entries\n";
 		$entry =~ s/^\s+//;
 		my $oui = substr $entry, 0, 8;
-		__PACKAGE__->add_to_cache( $oui, parse_oui( $entry ) );
+		$Cached->{ $oui } = parse_oui( $entry );
+		#last if $count > 100;
 		}
 
 	return 1;
@@ -499,56 +357,6 @@ sub load_cache {
 
 =back
 
-=head1 Caching
-
-Eventually I want people to write their own caching classes so I've
-created some class methods for this.
-
-=over 4
-
-=cut
-
-BEGIN {
-my $Cached = {};
-
-=item add_to_cache( OUI, PARSED_DATA )
-
-Add to the cache. This is mostly in place for a future expansion to
-full objects so you can override this in a subclass.
-
-=cut
-
-sub add_to_cache {
-	my( $class, $oui, $parsed ) = @_;
-
-	$Cached->{ $oui } = $parsed;
-	}
-
-=item get_from_cache( OUI )
-
-Get from the cache. This is mostly in place for a future expansion to
-full objects so you can override this in a subclass.
-
-=cut
-
-sub get_from_cache {
-	my( $class, $oui ) = @_;
-
-	$Cached->{ $oui };
-	}
-
-=item get_cache_hash()
-
-Get the hash the built-in cache uses. You should only use this if you
-were using the old C<$Cached> package variable.
-
-=cut
-
-sub get_cache_hash { $Cached }
-}
-
-=back
-
 =head1 SEE ALSO
 
 L<Net::MacMap>
@@ -565,7 +373,7 @@ brian d foy C<< <bdfoy@cpan.org> >>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright © 2004-2015, brian d foy C<< <bdfoy@cpan.org> >>. All rights reserved.
+Copyright (c) 2004-2012 brian d foy.  All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
